@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/ed.term.c,v 1.8 1991/12/19 22:34:14 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.02/RCS/ed.term.c,v 1.9 1992/03/27 01:59:46 christos Exp $ */
 /*
  * ed.term.c: Low level terminal interface
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: ed.term.c,v 1.8 1991/12/19 22:34:14 christos Exp $")
+RCSID("$Id: ed.term.c,v 1.9 1992/03/27 01:59:46 christos Exp $")
 
 #include "ed.h"
 #include "ed.term.h"
@@ -589,7 +589,7 @@ dosetty(v, t)
 		    xprintf("%s ", m->m_name);
 	    }
 	}
-	xprintf("\n");
+	xputchar('\n');
 	return;
     }
     while (v && (s = *v++)) {
@@ -1092,8 +1092,8 @@ tty_printchar(s)
 	if (m->m_name)
 	    xprintf("%s ^%c ", m->m_name, s[i] + 'A' - 1);
 	if (i % 5 == 0)
-	    xprintf("\n");
+	    xputchar('\n');
     }
-    xprintf("\n"); 
+    xputchar('\n');
 }
 #endif /* DEBUG_TTY */
