@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/sh.init.c,v 3.10 1991/12/14 20:45:46 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.init.c,v 3.15 1992/05/09 04:03:53 christos Exp $ */
 /*
  * sh.init.c: Function and signal tables
  */
@@ -36,9 +36,10 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.init.c,v 3.10 1991/12/14 20:45:46 christos Exp $")
+RCSID("$Id: sh.init.c,v 3.15 1992/05/09 04:03:53 christos Exp $")
 
 #include "ed.h"
+#include "tw.h"
 
 /*
  * C shell
@@ -47,153 +48,166 @@ RCSID("$Id: sh.init.c,v 3.10 1991/12/14 20:45:46 christos Exp $")
 #define	INF	0x7fffffff
 
 struct	biltins bfunc[] = {
-    { "@",	dolet,		0,	INF, },
-    { "alias",	doalias,	0,	INF, },
+    { "@",		dolet,		0,	INF, },
+    { "alias",		doalias,	0,	INF, },
     { "aliases",	doaliases,	0,	1, /* PWP */ },
-    { "alloc",	showall,	0,	1, },
-    { "bg",	dobg,		0,	INF, },
-    { "bind",	dobind,		0,	2, },
+    { "alloc",		showall,	0,	1, },
+    { "bg",		dobg,		0,	INF, },
+    { "bind",		dobind,		0,	2, },
     { "bindkey",	dobindkey,	0,	8, },
-    { "break",	dobreak,	0,	0, },
+    { "break",		dobreak,	0,	0, },
     { "breaksw",	doswbrk,	0,	0, },
 #if defined(IIASA) || defined(KAI)
-    { "bye",	goodbye,	0,	0, },
+    { "bye",		goodbye,	0,	0, },
 #endif
-    { "case",	dozip,		0,	1, },
-    { "cd",	dochngd,	0,	INF, },
-    { "chdir",	dochngd,	0,	INF, },
+    { "case",		dozip,		0,	1, },
+    { "cd",		dochngd,	0,	INF, },
+    { "chdir",		dochngd,	0,	INF, },
+    { "complete",	docomplete,	0,	INF, },
     { "continue",	docontin,	0,	0, },
     { "default",	dozip,		0,	0, },
-    { "dirs",	dodirs,		0,	INF, },
-    { "echo",	doecho,		0,	INF, },
-    { "echotc",	doechotc,	0,	INF, },
-    { "else",	doelse,		0,	INF, },
-    { "end",	doend,		0,	0, },
-    { "endif",	dozip,		0,	0, },
-    { "endsw",	dozip,		0,	0, },
-    { "eval",	doeval,		0,	INF, },
-    { "exec",	execash,	1,	INF, },
-    { "exit",	doexit,		0,	INF, },
-    { "fg",	dofg,		0,	INF, },
+    { "dirs",		dodirs,		0,	INF, },
+    { "echo",		doecho,		0,	INF, },
+    { "echotc",		doechotc,	0,	INF, },
+    { "else",		doelse,		0,	INF, },
+    { "end",		doend,		0,	0, },
+    { "endif",		dozip,		0,	0, },
+    { "endsw",		dozip,		0,	0, },
+    { "eval",		doeval,		0,	INF, },
+    { "exec",		execash,	1,	INF, },
+    { "exit",		doexit,		0,	INF, },
+    { "fg",		dofg,		0,	INF, },
     { "foreach",	doforeach,	3,	INF, },
 #ifdef TCF
     { "getspath",	dogetspath,	0,	0, },
-    { "getxvers", dogetxvers,	0,	0, },
+    { "getxvers",	 dogetxvers,	0,	0, },
 #endif /* TCF */
 #ifdef IIASA
-    { "gd",	dopushd,	0,	INF, },
+    { "gd",		dopushd,	0,	INF, },
 #endif
-    { "glob",	doglob,		0,	INF, },
-    { "goto",	dogoto,		1,	1, },
+    { "glob",		doglob,		0,	INF, },
+    { "goto",		dogoto,		1,	1, },
 #ifdef VFORK
     { "hashstat",	hashstat,	0,	0, },
 #endif
     { "history",	dohist,		0,	2, },
-    { "if",	doif,		1,	INF, },
+    { "if",		doif,		1,	INF, },
 #ifdef apollo
-    { "inlib",    doinlib,	1,	INF, },
+    { "inlib", 		doinlib,	1,	INF, },
 #endif
-    { "jobs",	dojobs,		0,	1, },
-    { "kill",	dokill,		1,	INF, },
+    { "jobs",		dojobs,		0,	1, },
+    { "kill",		dokill,		1,	INF, },
 #ifndef HAVENOLIMIT
-    { "limit",	dolimit,	0,	3, },
+    { "limit",		dolimit,	0,	3, },
 #endif /* ! HAVENOLIMIT */
     { "linedit",	doecho,		0,	INF, },
 #ifndef KAI
-    { "log",	dolog,		0,	0, },
+    { "log",		dolog,		0,	0, },
 #endif
-    { "login",	dologin,	0,	1, },
-    { "logout",	dologout,	0,	0, },
-    { "ls-F",	dolist,		0,	INF, },
+    { "login",		dologin,	0,	1, },
+    { "logout",		dologout,	0,	0, },
+    { "ls-F",		dolist,		0,	INF, },
 #ifdef TCF
     { "migrate",	domigrate,	1,	INF, },
 #endif /* TCF */
 #ifdef NEWGRP
-    { "newgrp",	donewgrp,	1,	1, },
+    { "newgrp",		donewgrp,	1,	1, },
 #endif
-    { "nice",	donice,		0,	INF, },
-    { "nohup",	donohup,	0,	INF, },
-    { "notify",	donotify,	0,	INF, },
-    { "onintr",	doonintr,	0,	2, },
-    { "popd",	dopopd,		0,	INF, },
-    { "pushd",	dopushd,	0,	INF, },
+    { "nice",		donice,		0,	INF, },
+    { "nohup",		donohup,	0,	INF, },
+    { "notify",		donotify,	0,	INF, },
+    { "onintr",		doonintr,	0,	2, },
+    { "popd",		dopopd,		0,	INF, },
+    { "pushd",		dopushd,	0,	INF, },
 #ifdef IIASA
-    { "rd",	dopopd,		0,	INF, },
+    { "rd",		dopopd,		0,	INF, },
 #endif
-    { "rehash",	dohash,		0,	3, },
-    { "repeat",	dorepeat,	2,	INF, },
+    { "rehash",		dohash,		0,	3, },
+    { "repeat",		dorepeat,	2,	INF, },
 #ifdef apollo
-    { "rootnode", dorootnode,	1,	1, },
+    { "rootnode",	dorootnode,	1,	1, },
 #endif
-    { "sched",	dosched,	0,	INF, },
-    { "set",	doset,		0,	INF, },
-    { "setenv",	dosetenv,	0,	2, },
+    { "sched",		dosched,	0,	INF, },
+    { "set",		doset,		0,	INF, },
+    { "setenv",		dosetenv,	0,	2, },
 #ifdef MACH
     { "setpath",	dosetpath,	0,	INF, },
 #endif	/* MACH */
 #ifdef TCF
     { "setspath",	dosetspath,	1,	INF, },
 #endif /* TCF */
-    { "settc",	dosettc,	2,	2, },
-    { "setty",  dosetty,	0,      INF },
+    { "settc",		dosettc,	2,	2, },
+    { "setty", 		dosetty,	0,      INF },
 #ifdef TCF
     { "setxvers",	dosetxvers,	0,	1, },
 #endif /* TCF */
-    { "shift",	shift,		0,	1, },
-    { "source",	dosource,	1,	2, },
-    { "stop",	dostop,		1,	INF, },
+    { "shift",		shift,		0,	1, },
+    { "source",		dosource,	1,	INF, },
+    { "stop",		dostop,		1,	INF, },
     { "suspend",	dosuspend,	0,	0, },
-    { "switch",	doswitch,	1,	INF, },
-    { "telltc",	dotelltc,	0,	INF, },
-    { "time",	dotime,		0,	INF, },
-    { "umask",	doumask,	0,	1, },
+    { "switch",		doswitch,	1,	INF, },
+    { "telltc",		dotelltc,	0,	INF, },
+    { "time",		dotime,		0,	INF, },
+    { "umask",		doumask,	0,	1, },
     { "unalias",	unalias,	1,	INF, },
-    { "unhash",	dounhash,	0,	0, },
+    { "uncomplete",	douncomplete,	1,	INF, },
+    { "unhash",		dounhash,	0,	0, },
 #ifdef masscomp
     { "universe",	douniverse,	0,	1, },
 #endif
 #ifndef HAVENOLIMIT
     { "unlimit",	dounlimit,	0,	INF, },
 #endif /* !HAVENOLIMIT */
-    { "unset",	unset,		1,	INF, },
+    { "unset",		unset,		1,	INF, },
     { "unsetenv",	dounsetenv,	1,	INF, },
 #ifdef apollo
-    { "ver",	dover,		0,	INF, },
+    { "ver",		dover,		0,	INF, },
 #endif
-    { "wait",	dowait,		0,	0, },
+    { "wait",		dowait,		0,	0, },
 #ifdef WARP
-    { "warp",	dowarp,		0,	2, },
+    { "warp",		dowarp,		0,	2, },
 #endif
 #ifdef KAI
     { "watchlog",	dolog,		0,	0, },
 #endif
-    { "where",	dowhere,	1,	INF, },
-    { "which",	dowhich,	1,	INF, },
-    { "while",	dowhile,	1,	INF, },
+    { "where",		dowhere,	1,	INF, },
+    { "which",		dowhich,	1,	INF, },
+    { "while",		dowhile,	1,	INF, },
 };
 int nbfunc = sizeof bfunc / sizeof *bfunc;
 
 struct srch srchn[] = {
-    { "@",	T_LET, },
-    { "break",	T_BREAK, },
+    { "@",		T_LET, },
+    { "break",		T_BREAK, },
     { "breaksw",	T_BRKSW, },
-    { "case",	T_CASE, },
+    { "case",		T_CASE, },
     { "default", 	T_DEFAULT, },
-    { "else",	T_ELSE, },
-    { "end",	T_END, },
-    { "endif",	T_ENDIF, },
-    { "endsw",	T_ENDSW, },
-    { "exit",	T_EXIT, },
+    { "else",		T_ELSE, },
+    { "end",		T_END, },
+    { "endif",		T_ENDIF, },
+    { "endsw",		T_ENDSW, },
+    { "exit",		T_EXIT, },
     { "foreach", 	T_FOREACH, },
-    { "goto",	T_GOTO, },
-    { "if",	T_IF, },
-    { "label",	T_LABEL, },
-    { "set",	T_SET, },
-    { "switch",	T_SWITCH, },
-    { "while",	T_WHILE, },
+    { "goto",		T_GOTO, },
+    { "if",		T_IF, },
+    { "label",		T_LABEL, },
+    { "set",		T_SET, },
+    { "switch",		T_SWITCH, },
+    { "while",		T_WHILE, },
 };
 int nsrchn = sizeof srchn / sizeof *srchn;
 
+#ifdef SUSPENDED
+# define MSG_STOP		"Suspended (signal)"
+# define MSG_TSTP		"Suspended"
+# define MSG_TTIN		"Suspended (tty input)"
+# define MSG_TTOU		"Suspended (tty output)"
+#else /* STOPPED */
+# define MSG_STOP		"Stopped (signal)"
+# define MSG_TSTP		"Stopped"
+# define MSG_TTIN		"Stopped (tty input)"
+# define MSG_TTOU		"Stopped (tty output)"
+#endif /* SUSPENDED */
 /*
  * Note: For some machines, (hpux eg.)
  * NSIG = number of signals + 1...
@@ -207,32 +221,43 @@ struct	mesg mesg[] = {
 /*  3 */	"QUIT",		"Quit",
 /*  4 */	"ILL",		"Illegal instruction",
 /*  5 */	"TRAP",		"Trace/BPT trap",
-#if SVID > 3
+#if SYSVREL > 3
 /*  6 */	"ABRT",		"Abort",
-#else /* SVID > 3 */
+#else /* SYSVREL > 3 */
 /*  6 */	"IOT",		"IOT trap",
-#endif /* SVID > 3 */
+#endif /* SYSVREL > 3 */
 #ifdef aiws
 /*  7 */	"DANGER", 	"System Crash Imminent",
 #else /* aiws */
+# ifdef linux
+/*  7 */	0,		"Signal 7",
+# else /* linux */
 /*  7 */	"EMT",		"EMT trap",
+# endif /* linux */
 #endif /* aiws */
 /*  8 */	"FPE",		"Floating exception",
 /*  9 */	"KILL",		"Killed",
+#ifdef linux
+/* 10 */	"USR1",		"User signal 1",
+/* 11 */	"SEGV",		"Segmentation fault",
+/* 12 */	"USR2",		"User signal 2",
+#else /* linux */
 /* 10 */	"BUS",		"Bus error",
 /* 11 */	"SEGV",		"Segmentation fault",
 /* 12 */	"SYS",		"Bad system call",
+#endif /* linux */
 /* 13 */	"PIPE",		"Broken pipe",
 /* 14 */	"ALRM",		"Alarm clock",
 /* 15 */	"TERM",		"Terminated",
 
-#if (SVID > 0) || defined(DGUX) || defined(IBMAIX) || defined(apollo)
+
+#if (SYSVREL > 0) || defined(DGUX) || defined(IBMAIX) || defined(apollo) || defined(masscomp) || defined(ardent)
 
 # ifdef _sigextra_
 #  undef  _sigextra_
 # endif /* _sigextra_ */
 
-#if !defined(IBMAIX) && !defined(cray)
+#if !defined(IBMAIX) && !defined(cray) && !defined(linux)
 /* these are the real svid signals */
 /* 16 */	"USR1",		"User signal 1",
 /* 17 */	"USR2", 	"User signal 2",
@@ -243,7 +268,28 @@ struct	mesg mesg[] = {
 /* 18 */	"CHLD",		"Child exited",
 /* 19 */	"PWR",  	"Power failure",
 # endif /* apollo */
-#endif /* IBMAIX */
+#endif /* IBMAIX && cray && linux */
+
+#ifdef linux
+# define _sigextra_
+/* 16 */	0,		"Signal 16",
+/* 17 */	"CHLD",		"Child exited",
+/* 18 */	"CONT",		"Continued",
+/* 19 */	"STOP",		MSG_STOP,
+/* 20 */	"TSTP",		MSG_TSTP,
+/* 21 */	"TTIN", 	MSG_TTIN,
+/* 22 */	"TTOU", 	MSG_TTOU,
+/* 23 */	0,   		"Signal 23",
+/* 24 */	0,		"Signal 24",
+/* 25 */	0, 		"Signal 25",
+/* 26 */	0,	 	"Signal 26",
+/* 27 */	0,		"Signal 27",
+/* 28 */	"WINCH", 	"Window changed",
+/* 29 */	0,		"Signal 29",
+/* 30 */	0,		"Signal 30",
+/* 31 */	0,		"Signal 31",
+/* 32 */	0,		"Signal 32",
+#endif /* linux */
 
 # ifdef cray
 # define _sigextra_
@@ -259,21 +305,11 @@ struct	mesg mesg[] = {
 /* 25 */	"DLK",		"True deadlock detected",
 /* 26 */	"CPULIM",	"CPU time limit exceeded",
 /* 27 */	"SHUTDN",	"System shutdown imminent",
-# ifdef SUSPENDED
-/* 28 */	"STOP", 	"Suspended",
-/* 29 */	"TSTP", 	"Suspended",
-# else /* SUSPENDED */
-/* 28 */	"STOP", 	"Stopped",
-/* 29 */	"TSTP", 	"Stopped",
-# endif /* SUSPENDED */
+/* 28 */	"STOP", 	MSG_STOP,
+/* 29 */	"TSTP", 	MSG_TSTP,
 /* 30 */	"CONT",   	"Continue",
-# ifdef SUSPENDED
-/* 31 */	"TTIN",		"Suspended (tty input)",
-/* 32 */	"TTOU",		"Suspended (tty output)",
-# else /* SUSPENDED */
-/* 31 */	"TTIN",		"Stopped (tty input)",
-/* 32 */	"TTOU",		"Stopped (tty output)",
-# endif /* SUSPENDED */
+/* 31 */	"TTIN",		MSG_TTIN,
+/* 32 */	"TTOU",		MSG_TTOU,
 /* 33 */	"WINCH",	"Window size changed",
 /* 34 */	"RPE",		"CRAY Y-MP register parity error",
 /* 35 */	0,		"Signal 35",
@@ -318,17 +354,10 @@ struct	mesg mesg[] = {
 
 # ifdef OREO
 #  define _sigextra_
-#  ifdef SUSPENDED
-/* 20 */	"TSTP",		"Suspended",
-/* 21 */	"TTIN", 	"Suspended (tty input)",
-/* 22 */	"TTOU", 	"Suspended (tty output)",
-/* 23 */	"STOP",		"Suspended (signal)",
-#  else
-/* 20 */	"TSTP",		"Stopped",
-/* 21 */	"TTIN", 	"Stopped (tty input)",
-/* 22 */	"TTOU", 	"Stopped (tty output)",
-/* 23 */	"STOP",		"Stopped (signal)",
-#  endif /* SUSPENDED */
+/* 20 */	"TSTP",		MSG_TSTP,
+/* 21 */	"TTIN", 	MSG_TTIN,
+/* 22 */	"TTOU", 	MSG_TTOU,
+/* 23 */	"STOP",		MSG_STOP,
 /* 24 */	"XCPU",		"Cputime limit exceeded",
 /* 25 */	"XFSZ", 	"Filesize limit exceeded",
 /* 26 */	"VTALRM", 	"Virtual time alarm",
@@ -346,21 +375,11 @@ struct	mesg mesg[] = {
 /* 21 */	"PROF", 	"Profiling time alarm",
 /* 22 */	"IO", 		"Asynchronous I/O (select)",
 /* 23 */	"WINDOW", 	"Window changed",
-#  ifdef SUSPENDED
-/* 24 */	"STOP",		"Suspended (signal)",
-/* 25 */	"TSTP",		"Suspended",
-#  else /* SUSPENDED */
-/* 24 */	"STOP",		"Stopped (signal)",
-/* 25 */	"TSTP",		"Stopped",
-#  endif /* SUSPENDED */
+/* 24 */	"STOP",		MSG_STOP,
+/* 25 */	"TSTP",		MSG_TSTP,
 /* 26 */	"CONT",		"Continued",
-#  ifdef SUSPENDED
-/* 27 */	"TTIN", 	"Suspended (tty input)",
-/* 28 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 27 */	"TTIN", 	"Stopped (tty input)",
-/* 28 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 27 */	"TTIN", 	MSG_TTIN,
+/* 28 */	"TTOU", 	MSG_TTOU,
 /* 29 */	"URG",		"Urgent condition on IO channel",
 /* 30 */	"LOST",		"Remote lock lost (NFS)",
 /* 31 */	0, 		"Reserved", /* Reserved */
@@ -372,21 +391,11 @@ struct	mesg mesg[] = {
 /* 20 */	"WINDOW", 	"Window changed",
 /* 21 */	"URG",		"Urgent condition on IO channel",
 /* 22 */	"POLL", 	"Pollable event occured",
-#  ifdef SUSPENDED
-/* 23 */	"STOP",		"Suspended (signal)",
-/* 24 */	"TSTP",		"Suspended",
-#  else /* SUSPENDED */
-/* 23 */	"STOP",		"Stopped (signal)",
-/* 24 */	"TSTP",		"Stopped",
-#  endif /* SUSPENDED */
+/* 23 */	"STOP",		MSG_STOP,
+/* 24 */	"TSTP",		MSG_TSTP,
 /* 25 */	"CONT",		"Continued",
-#  ifdef SUSPENDED
-/* 26 */	"TTIN", 	"Suspended (tty input)",
-/* 27 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 26 */	"TTIN", 	"Stopped (tty input)",
-/* 27 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 26 */	"TTIN", 	MSG_TTIN,
+/* 27 */	"TTOU", 	MSG_TTOU,
 /* 28 */	"IO", 		"Asynchronous I/O (select)",
 /* 29 */	"XCPU",		"Cputime limit exceeded",
 /* 30 */	"XFSZ", 	"Filesize limit exceeded",
@@ -394,47 +403,48 @@ struct	mesg mesg[] = {
 /* 32 */	"PROF", 	"Profiling time alarm",
 # endif /* stellar */
 
-# if SVID > 3
+# ifdef ardent
+#  define _sigextra_
+/* 20 */	"WINDOW", 	"Window changed",
+/* 21 */	"URG",		"Urgent condition on IO channel",
+/* 22 */	"POLL", 	"Pollable event occured",
+/* 23 */	"STOP",		MSG_STOP,
+/* 24 */	"TSTP",		MSG_TSTP,
+/* 25 */	"TTIN", 	MSG_TTIN,
+/* 26 */	"TTOU", 	MSG_TTOU,
+/* 27 */	"CONT",		"Continued",
+/* 28 */	"XCPU",		"Cputime limit exceeded",
+/* 29 */	"XFSZ", 	"Filesize limit exceeded",
+/* 30 */	"VTALRM", 	"Virtual time alarm",
+/* 31 */	"PROF", 	"Profiling time alarm",
+# endif /* ardent */
+
+# if SYSVREL > 3
 #  define _sigextra_
 /* 20 */	"WINCH", 	"Window change",
 /* 21 */	"URG", 		"Urgent socket condition",
 /* 22 */	"IO", 		"Socket I/O possible",
-#  ifdef SUSPENDED
-/* 23 */	"STOP",		"Suspended (signal)",
-/* 24 */	"TSTP",		"Suspended",
+/* 23 */	"STOP",		MSG_STOP,
+/* 24 */	"TSTP",		MSG_TSTP,
 /* 25 */	"CONT",		"Continued",
-/* 26 */	"TTIN", 	"Suspended (tty input)",
-/* 27 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 23 */	"STOP",		"Stopped (signal)",
-/* 24 */	"TSTP",		"Stopped",
-/* 25 */	"CONT",		"Continued",
-/* 26 */	"TTIN", 	"Stopped (tty input)",
-/* 27 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 26 */	"TTIN", 	MSG_TTIN,
+/* 27 */	"TTOU", 	MSG_TTOU,
 /* 28 */	"VTALRM",	"Virtual timer expired",
 /* 29 */	"PROF",		"Profiling timer expired",
 /* 30 */	"XCPU",		"CPU time limit exceeded",
 /* 31 */	"XFSZ", 	"File size limit exceeded",
 /* 32 */	0,		"Maximum number of signals",
-# endif /* SVID > 3 */
+# endif /* SYSVREL > 3 */
 # if defined(ISC) && defined(POSIX) 
 #  define _sigextra_
 /* 20 */	"WINCH", 	"Window change",
 /* 21 */	0, 		"Unused", /* SIGPHONE used only for UNIXPC */
 /* 22 */	"POLL", 	"Pollable event occured",
 /* 23 */	"CONT", 	"Continued",
-#  ifdef SUSPENDED
-/* 24 */	"STOP",		"Suspended (signal)",
-/* 25 */	"TSTP",		"Suspended",
-/* 26 */	"TTIN", 	"Suspended (tty input)",
-/* 27 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 24 */	"STOP",		"Stopped (signal)",
-/* 25 */	"TSTP",		"Stopped",
-/* 26 */	"TTIN", 	"Stopped (tty input)",
-/* 27 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 24 */	"STOP",		MSG_STOP,
+/* 25 */	"TSTP",		MSG_TSTP,
+/* 26 */	"TTIN", 	MSG_TTIN,
+/* 27 */	"TTOU", 	MSG_TTOU,
 /* 28 */	0,	  	"number of signals",
 /* 29 */	0,		"Reserved", /* Reserved */
 /* 30 */	0,		"Reserved", /* Reserved */
@@ -447,19 +457,11 @@ struct	mesg mesg[] = {
 /* 20 */	"WINCH", 	"Window change",
 /* 21 */	0, 		"Unused", /* SIGPHONE used only for UNIXPC */
 /* 22 */	"POLL", 	"Pollable event occured",
-#  ifdef SUSPENDED
-/* 23 */	"STOP",		"Suspended (signal)",
-/* 24 */	"TSTP",		"Suspended",
+/* 23 */	"STOP",		MSG_STOP,
+/* 24 */	"TSTP",		MSG_TSTP,
 /* 25 */	"CONT", 	"Continued",
-/* 26 */	"TTIN", 	"Suspended (tty input)",
-/* 27 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 23 */	"STOP",		"Stopped (signal)",
-/* 24 */	"TSTP",		"Stopped",
-/* 25 */	"CONT", 	"Continued",
-/* 26 */	"TTIN", 	"Stopped (tty input)",
-/* 27 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 26 */	"TTIN", 	MSG_TTIN,
+/* 27 */	"TTOU", 	MSG_TTOU,
 /* 28 */	0,	  	"number of signals",
 /* 29 */	0,		"Reserved", /* Reserved */
 /* 30 */	0,		"Reserved", /* Reserved */
@@ -469,13 +471,8 @@ struct	mesg mesg[] = {
 
 # ifdef IRIS4D
 #  define _sigextra_
-#  ifdef SUSPENDED
-/* 20 */	"STOP",		"Suspended (signal)",
-/* 21 */	"TSTP",		"Suspended",
-#  else /* SUSPENDED */
-/* 20 */	"STOP",		"Stopped (signal)",
-/* 21 */	"TSTP",		"Stopped",
-#  endif /* SUSPENDED */
+/* 20 */	"STOP",		MSG_STOP,
+/* 21 */	"TSTP",		MSG_TSTP,
 /* 22 */	"POLL", 	"Stream I/O pending",
 /* 23 */	"IO", 		"Asynchronous I/O (select)",
 /* 24 */	"URG",		"Urgent condition on IO channel",
@@ -483,13 +480,8 @@ struct	mesg mesg[] = {
 /* 26 */	"VTALRM", 	"Virtual time alarm",
 /* 27 */	"PROF", 	"Profiling time alarm",
 /* 28 */	"CONT",		"Continued",
-#  ifdef SUSPENDED
-/* 29 */	"TTIN", 	"Suspended (tty input)",
-/* 30 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 29 */	"TTIN", 	"Stopped (tty input)",
-/* 30 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 29 */	"TTIN", 	MSG_TTIN,
+/* 30 */	"TTOU", 	MSG_TTOU,
 /* 31 */	0,		"Signal 31",
 /* 32 */	0,		"Signal 32",
 # endif /* IRIS4D */
@@ -513,22 +505,12 @@ struct	mesg mesg[] = {
 
 # ifdef apollo
 #  define _sigextra_
-#  ifdef SUSPENDED
-/* 20 */	"STOP",		"Suspended (signal)",
-/* 21 */	"TSTP",		"Suspended",
-#  else /* SUSPENDED */
-/* 20 */	"STOP",		"Stopped (signal)",
-/* 21 */	"TSTP",		"Stopped",
-#  endif /* SUSPENDED */
+/* 20 */	"STOP",		MSG_STOP,
+/* 21 */	"TSTP",		MSG_TSTP,
 /* 22 */	"CONT",		"Continued",
 /* 23 */	"CHLD",		"Child stopped or exited",
-#  ifdef SUSPENDED
-/* 24 */	"TTIN", 	"Suspended (tty input)",
-/* 25 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 24 */	"TTIN", 	"Stopped (tty input)",
-/* 25 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 24 */	"TTIN", 	MSG_TTIN,
+/* 25 */	"TTOU", 	MSG_TTOU,
 /* 26 */	"IO", 		"Asynchronous I/O (select)",
 /* 27 */	"XCPU",		"Cputime limit exceeded",
 /* 28 */	"XFSZ", 	"Filesize limit exceeded",
@@ -538,6 +520,23 @@ struct	mesg mesg[] = {
 /* 32 */	"WINCH", 	"Window changed",
 # endif /* apollo */
 
+# ifdef masscomp
+#  define _sigextra_
+/* 20 */	"STOP",		MSG_STOP,
+/* 21 */	"TSTP",		MSG_TSTP,
+/* 22 */	"CONT",		"Continued",
+/* 23 */	"TTIN", 	MSG_TTIN,
+/* 24 */	"TTOU", 	MSG_TTOU,
+/* 25 */	"TINT", 	"New input character",
+/* 26 */	"XCPU",		"Cputime limit exceeded",
+/* 27 */	"XFSZ", 	"Filesize limit exceeded",
+/* 28 */	"WINCH", 	"Window changed",
+/* 29 */	"URG",		"Urgent condition on IO channel",
+/* 30 */	"VTALRM", 	"Virtual time alarm",
+/* 31 */	"PROF", 	"Profiling time alarm",
+/* 32 */	"IO", 		"Asynchronous I/O (select)",
+# endif /* masscomp */
+
 # ifdef aiws
 #  define _sigextra_
 /* 20 */	0,		"Signal 20",
@@ -548,41 +547,23 @@ struct	mesg mesg[] = {
 /* 25 */	"IOINT", 	"I/O intervention required",
 /* 26 */	"GRANT", 	"monitor mode granted",
 /* 27 */	"RETRACT", 	"monitor mode retracted",
-/* 28 */	"WINCH","Window size changed",
+/* 28 */	"WINCH",	"Window size changed",
 /* 29 */	0,		"Signal 29",
 /* 30 */	"SOUND", 	"sound completed",
 /* 31 */	"MSG", 		"input hft data pending",
 /* 32 */	0,		"Signal 32",
 # endif /* aiws */
 
-# ifdef m88k				/* Motorola 88100: POSIX/BCS signals */
+# if defined(m88k) || defined(__m88k__)	/* Motorola 88100: POSIX/BCS signals */
 #  define _sigextra_
 /* 20 */	"WINCH", 	"Window changed",
-# ifdef DGUX
 /* 21 */	0,		"Signal 21",
-# else
-#  ifdef SUSPENDED
-/* 21 */	"TTIN", 	"Suspended (tty input)",
-#  else /* SUSPENDED */
-/* 21 */	"TTIN", 	"Stopped (tty input)",
-#  endif /* SUSPENDED */
-# endif /* DGUX */
 /* 22 */	"POLL", 	"Stream I/O pending",
-# ifdef SUSPENDED
-/* 23 */	"STOP",		"Suspended (signal)",
-/* 24 */	"TSTP",		"Suspended",
-# else /* SUSPENDED */
-/* 23 */	"STOP",		"Stopped (signal)",
-/* 24 */	"TSTP",		"Stopped",
-# endif /* SUSPENDED */
+/* 23 */	"STOP",		MSG_STOP,
+/* 24 */	"TSTP",		MSG_TSTP,
 /* 25 */	"CONT",		"Continued",
-# ifdef SUSPENDED
-/* 26 */	"TTIN", 	"Suspended (tty input)",
-/* 27 */	"TTOU", 	"Suspended (tty output)",
-# else /* SUSPENDED */
-/* 26 */	"TTIN", 	"Stopped (tty input)",
-/* 27 */	"TTOU", 	"Stopped (tty output)",
-# endif /* SUSPENDED */
+/* 26 */	"TTIN", 	MSG_TTIN,
+/* 27 */	"TTOU", 	MSG_TTOU,
 /* 28 */	0,		"Signal 28",
 /* 29 */	0,		"Signal 29",
 /* 30 */	0,		"Signal 30",
@@ -620,29 +601,17 @@ struct	mesg mesg[] = {
 /* 62 */	0,		"Signal 62",
 /* 63 */	0,		"Signal 63",
 /* 64 */	0,		"Signal 64",
-# endif /* m88k */
-
+# endif /* m88k || __m88k__ */
 
 #ifdef IBMAIX
 # define _sigextra_
-
 /* 16 */	"URG",		"Urgent condition on IO channel",
-# ifdef SUSPENDED
-/* 17 */	"STOP",		"Suspended (signal)",
-/* 18 */	"TSTP",		"Suspended",
-# else /* SUSPENDED */
-/* 17 */	"STOP",		"Stopped (signal)",
-/* 18 */	"TSTP",		"Stopped",
-# endif /* SUSPENDED */
+/* 17 */	"STOP",		MSG_STOP,
+/* 18 */	"TSTP",		MSG_TSTP,
 /* 19 */	"CONT",		"Continued",
 /* 20 */	"CHLD",		"Child exited",
-# ifdef SUSPENDED
-/* 21 */	"TTIN", 	"Suspended (tty input)",
-/* 22 */	"TTOU", 	"Suspended (tty output)",
-# else /* SUSPENDED */
-/* 21 */	"TTIN", 	"Stopped (tty input)",
-/* 22 */	"TTOU", 	"Stopped (tty output)",
-# endif /* SUSPENDED */
+/* 21 */	"TTIN", 	MSG_TTIN,
+/* 22 */	"TTOU", 	MSG_TTOU,
 /* 23 */	"IO",   	"IO possible interrupt",
 /* 24 */	"XCPU",		"Cputime limit exceeded",
 /* 25 */	"XFSZ", 	"Filesize limit exceeded",
@@ -691,24 +660,17 @@ struct	mesg mesg[] = {
 /* 64 */	0,		"Signal 64",
 #endif /* IBMAIX */
 
+
 # ifdef _SEQUENT_
 #  define _sigextra_
 /* 20 */	"WINCH", 	"Window changed",
 /* 21 */	0,		"Signal 21",
 /* 22 */	"POLL", 	"Stream I/O pending",
-#  ifdef SUSPENDED
-/* 23 */	"STOP",		"Suspended (signal)",
-/* 24 */	"TSTP",		"Suspended",
+/* 23 */	"STOP",		MSG_STOP,
+/* 24 */	"TSTP",		MSG_TSTP,
 /* 25 */	"CONT",		"Continued",
-/* 26 */	"TTIN", 	"Suspended (tty input)",
-/* 27 */	"TTOU", 	"Suspended (tty output)",
-#  else /* SUSPENDED */
-/* 23 */	"STOP",		"Stopped (signal)",
-/* 24 */	"TSTP",		"Stopped",
-/* 25 */	"CONT",		"Continued",
-/* 26 */	"TTIN", 	"Stopped (tty input)",
-/* 27 */	"TTOU", 	"Stopped (tty output)",
-#  endif /* SUSPENDED */
+/* 26 */	"TTIN", 	MSG_TTIN,
+/* 27 */	"TTOU", 	MSG_TTOU,
 /* 28 */	0, 		"Signal 28",
 /* 29 */	0,		"Signal 29",
 /* 30 */	0, 		"Signal 30",
@@ -740,29 +702,28 @@ struct	mesg mesg[] = {
 # endif /* _sigextra_ */
 
 /* 16 */	"URG",		"Urgent condition on IO channel",
-# ifdef SUSPENDED
-/* 17 */	"STOP",		"Suspended (signal)",
-/* 18 */	"TSTP",		"Suspended",
-# else /* SUSPENDED */
-/* 17 */	"STOP",		"Stopped (signal)",
-/* 18 */	"TSTP",		"Stopped",
-# endif /* SUSPENDED */
+/* 17 */	"STOP",		MSG_STOP,
+/* 18 */	"TSTP",		MSG_TSTP,
 /* 19 */	"CONT",		"Continued",
 /* 20 */	"CHLD",		"Child exited",
-# ifdef SUSPENDED
-/* 21 */	"TTIN", 	"Suspended (tty input)",
-/* 22 */	"TTOU", 	"Suspended (tty output)",
-# else /* SUSPENDED */
-/* 21 */	"TTIN", 	"Stopped (tty input)",
-/* 22 */	"TTOU", 	"Stopped (tty output)",
-# endif /* SUSPENDED */
+/* 21 */	"TTIN", 	MSG_TTIN,
+/* 22 */	"TTOU", 	MSG_TTOU,
 /* 23 */	"IO",   	"IO possible interrupt",
 /* 24 */	"XCPU",		"Cputime limit exceeded",
 /* 25 */	"XFSZ", 	"Filesize limit exceeded",
 /* 26 */	"VTALRM", 	"Virtual time alarm",
 /* 27 */	"PROF",		"Profiling time alarm",
 
-# if defined(sun) || defined(ultrix) || defined(hp9000) || defined(convex) || defined(__convex__)
+# if defined(RENO) || defined(BSD4_4)
+# define _sigextra_
+/* 28 */	"WINCH",	"Window size changed",
+/* 29 */	"INFO",		"Information request",
+/* 30 */	"USR1",		"User defined signal 1",
+/* 31 */	"USR2",		"User defined signal 2",
+/* 32 */	0,		"Signal 32",
+# endif /* RENO || BSD4_4 */
+
+# if (defined(sun) || defined(ultrix) || defined(hp9000) || defined(convex) || defined(__convex__)) && !defined(_sigextra_)
 #  define _sigextra_
 /* 28 */	"WINCH", 	"Window changed",
 /* 29 */	"LOST",		"Resource lost",
@@ -782,18 +743,13 @@ struct	mesg mesg[] = {
 
 # ifndef _sigextra_
 /* 28 */	"WINCH",	"Window size changed",
-#  ifdef RENO
-/* 29 */	"INFO",		"Information request",
-#  else
 /* 29 */	0,		"Signal 29",
-#  endif /* RENO */
 /* 30 */	"USR1",		"User defined signal 1",
 /* 31 */	"USR2",		"User defined signal 2",
 /* 32 */	0,		"Signal 32",
 # endif /* _sigextra_ */
 
-
-#endif /* (SVID > 0) || DGUX || IBMAIX */
+#endif /* (SYSVREL > 0) || DGUX || IBMAIX */
 
 /* These are here for systems with bad NSIG */
 #ifndef POSIX
