@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/sh.time.c,v 3.4 1991/11/26 04:41:23 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.time.c,v 3.7 1992/04/03 22:15:14 christos Exp $ */
 /*
  * sh.time.c: Shell time keeping and printing.
  */
@@ -36,7 +36,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: sh.time.c,v 3.4 1991/11/26 04:41:23 christos Exp $")
+RCSID("$Id: sh.time.c,v 3.7 1992/04/03 22:15:14 christos Exp $")
 
 #if defined(sun) && ! defined(MACH)
 # include <machine/param.h>
@@ -61,6 +61,7 @@ static	void	pdtimet	__P((clock_t, clock_t));
 static	void	pdtimet	__P((time_t, time_t));
 # endif /* ! POSIX */
 #else /* BSDTIMES || _SEQUENT_ */
+static 	void 	tvadd	__P((timeval_t *, timeval_t *));
 static	void	pdeltat	__P((timeval_t *, timeval_t *));
 #endif /* BSDTIMES || _SEQUENT_ */
 
@@ -298,7 +299,7 @@ prusage(bs, es, e, b)
 
 #  else /* POSIX */
     register clock_t t = (es->tms_utime - bs->tms_utime +
-			  es->tms_stime - bs->tms_stime) * 100 / CLK_TCK;
+			  es->tms_stime - bs->tms_stime) * 100 / clk_tck;
 
 #  endif /* POSIX */
 # endif /* _SEQUENT_ */
@@ -324,7 +325,7 @@ prusage(bs, es, e, b)
     time_t  ms = (e - b) * 100 / HZ;
 
 #  else /* POSIX */
-    clock_t ms = (e - b) * 100 / CLK_TCK;
+    clock_t ms = (e - b) * 100 / clk_tck;
 
 #  endif /* POSIX */
     cp = "%Uu %Ss %E %P";
@@ -561,7 +562,7 @@ pdeltat(t1, t0)
     xprintf("%ld.%03ld", td.tv_sec, td.tv_usec / 1000L);
 }
 
-void
+static void
 tvadd(tsum, t0)
     timeval_t *tsum, *t0;
 {
@@ -605,7 +606,7 @@ pdtimet(eval, bval)
 #ifndef POSIX
     val = (eval - bval) * 100 / HZ;
 #else /* POSIX */
-    val = (eval - bval) * 100 / CLK_TCK;
+    val = (eval - bval) * 100 / clk_tck;
 #endif /* POSIX */
 
     xprintf("%ld.%02ld", val / 100, val - (val / 100 * 100));
