@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/sh.types.h,v 3.16 1991/12/14 20:45:46 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.02/RCS/sh.types.h,v 3.19 1992/04/03 22:15:14 christos Exp $ */
 /* sh.types.h: Do the necessary typedefs for each system.
  *             Up till now I avoided making this into a separate file
  *	       But I just wanted to eliminate the whole mess from sh.h
@@ -78,14 +78,16 @@
 #  define _SPEED_T
 # endif /* _SPEED_T */
 # ifndef SUNOS4
-#  ifndef _UID_T
-#   define _UID_T
-     typedef int uid_t;
-#  endif /* _UID_T */
-#  ifndef _GID_T
-#   define _GID_T
-     typedef int gid_t;
-#  endif /* _GID_T */
+#  ifndef MACH
+#   ifndef _UID_T
+#    define _UID_T
+      typedef int uid_t;
+#   endif /* _UID_T */
+#   ifndef _GID_T
+#    define _GID_T
+      typedef int gid_t;
+#   endif /* _GID_T */
+#  endif /* !MACH */
 # endif /* !SUNOS4 */
 #endif /* sun */
 
@@ -170,20 +172,20 @@ typedef char * caddr_t;
  * DGUX types
  */
 # ifdef ___int_size_t_h
-#  ifdef _TARGETTING_M88KBCS_OR_DGUX
-#   ifdef _USING_ANSI_C_OR_POSIX_OR_SYSV3_OR_BSD_OR_DGUX
+#  if defined(_TARGETTING_M88KBCS_OR_DGUX) || defined(_TARGETTING_M88KBCS_OR_M88KOCS_OR_DGUX)
+#   if defined(_USING_ANSI_C_OR_POSIX_OR_SYSV3_OR_BSD_OR_DGUX) || defined(_USING_ANSI_C_OR_POSIX_OR_XPG3_OR_SYSV3_OR_BSD_OR_DGUX)
 #    ifndef _SIZE_T
 #     define _SIZE_T
 #    endif /* _SIZE_T */
-#   endif  /* #ifdef _USING_ANSI_C_OR_POSIX_OR_SYSV3_OR_BSD_OR_DGUX */
-#  endif  /* #ifdef _TARGETTING_M88KBCS_OR_DGUX */
-# endif  /* #ifndef ___int_size_t_h */
+#   endif  /* _USING_ANSI_C_OR_POSIX_OR_SYSV3_OR_BSD_OR_DGUX || _USING_ANSI_C_OR_POSIX_OR_XPG3_OR_SYSV3_OR_BSD_OR_DGUX */
+#  endif  /* _TARGETTING_M88KBCS_OR_DGUX || _TARGETTING_M88KBCS_OR_M88KOCS_OR_DGUX */
+# endif  /* ___int_size_t_h */
 
 # ifdef _USING_POSIX_OR_SYSV3_OR_BSD_OR_DGUX
 #  ifndef _PID_T
 #   define _PID_T
 #  endif /* _PID_T */
-# endif  /* #ifdef _USING_POSIX_OR_SYSV3_OR_BSD_OR_DGUX */
+# endif  /* _USING_POSIX_OR_SYSV3_OR_BSD_OR_DGUX */
 
 #endif
 
@@ -320,13 +322,22 @@ typedef char * caddr_t;
 #endif /* INTEL */
 
 /***
- *** Concurrent (Masscomp) running RTU 4.1A & RTU 5.0.
+ *** Concurrent (Masscomp) running RTU 4.1A & RTU 5.0. 
+ **** [RTU 6.0 from mike connor]
  *** Added, DAS DEC-90.
  ***/
 #ifdef	masscomp
 # ifndef _SIZE_T
 #  define _SIZE_T
 # endif /* _SIZE_T */
+# ifdef RTU6
+#  ifndef _PID_T
+#   define _PID_T
+#  endif /* _PID_T */
+#  ifndef _SPEED_T
+#   define _SPEED_T
+#  endif /* _SPEED_T */
+#endif /* RTU6 */
 #endif	/* masscomp */
 
 /***
@@ -410,6 +421,10 @@ typedef char * caddr_t;
 
 # ifndef _PID_T
 #  define _PID_T
+#if defined(i860) && defined(alliant)                 /* FX-2800 */
+# define _PID_T
+   typedef short pid_t;
+# endif       /* FX-2800 */
     typedef int pid_t;
 # endif /* _PID_T */
 
