@@ -1,4 +1,4 @@
-/* $Header: /home/hyperion/mu/christos/src/sys/tcsh-6.01/RCS/tc.wait.h,v 3.3 1991/11/26 04:28:26 christos Exp $ */
+/* $Header: /u/christos/src/tcsh-6.02/RCS/tc.wait.h,v 3.5 1992/05/15 23:49:22 christos Exp $ */
 /*
  * tc.wait.h: <sys/wait.h> for machines that don't have it or have it and
  *	      is incorrect.
@@ -43,7 +43,7 @@
  * We try to use the system's wait.h when we can...
  */
 
-#if SVID > 0
+#if SYSVREL > 0
 # ifdef hpux
 #  ifndef __hpux
 #   define NEEDwait
@@ -63,14 +63,14 @@
 #   define NEEDwait
 #  endif /* OREO || IRIS4D || POSIX */
 # endif	/* hpux */
-#else /* SVID == 0 */
-# include <sys/wait.h>
-#endif /* SVID == 0 */
-
-#ifdef _MINIX
-# undef NEEDwait
-# include "mi.wait.h"
-#endif /* _MINIX */
+#else /* SYSVREL == 0 */
+# ifdef _MINIX
+#  undef NEEDwait
+#  include "mi.wait.h"
+# else
+#  include <sys/wait.h>
+# endif /* _MINIX */
+#endif /* SYSVREL == 0 */
 
 #ifdef NEEDwait
 /*
